@@ -55,9 +55,9 @@
 typedef struct proc	Proc;
 struct proc {
 	Proc *next;		/* next process in pipeline (if any) */
+	pid_t pid;		/* process id */
 	int state;
 	int status;		/* wait status */
-	pid_t pid;		/* process id */
 	char command[48];	/* process command string */
 };
 
@@ -89,24 +89,22 @@ struct proc {
 typedef struct job Job;
 struct job {
 	Job *next;		/* next job in list */
+	Proc *proc_list;	/* process list */
+	Proc *last_proc;	/* last process in list */
+	struct timeval systime;	/* system time used by job */
+	struct timeval usrtime;	/* user time used by job */
+	pid_t pgrp;		/* process group of job */
+	pid_t ppid;		/* pid of process that forked job */
 	int job;		/* job number: %n */
 	int flags;		/* see JF_* */
 	volatile int state;	/* job state */
 	int status;		/* exit status of last process */
-	pid_t pgrp;		/* process group of job */
-	pid_t ppid;		/* pid of process that forked job */
 	int32_t	age;		/* number of jobs started */
-	struct timeval systime;	/* system time used by job */
-	struct timeval usrtime;	/* user time used by job */
-	Proc *proc_list;	/* process list */
-	Proc *last_proc;	/* last process in list */
-#ifdef KSH
+#ifdef SILLY_FEATURES
 	Coproc_id coproc_id;	/* 0 or id of coprocess output pipe */
-#endif /* KSH */
-#ifdef TTY_PGRP
-	TTY_state ttystate;	/* saved tty state for stopped jobs */
+	struct termios ttystate;/* saved tty state for stopped jobs */
 	pid_t saved_ttypgrp;	/* saved tty process group for stopped jobs */
-#endif /* TTY_PGRP */
+#endif
 };
 
 /* Flags for j_waitj() */
@@ -136,40 +134,25 @@ static pid_t async_pid;
 
 static int nzombie;		/* # of zombies owned by this process */
 static int32_t njobs;		/* # of jobs started */
-static int child_max;	/* CHILD_MAX */
+static long child_max;	/* CHILD_MAX */
 
 
+#ifdef JOB_SIGS
 /* held_sigchld is set if sigchld occurs before a job is completely started */
 static volatile sig_atomic_t held_sigchld;
+#endif /* JOB_SIGS */
 
-#ifdef JOBS
+#ifdef SILLY_FEATURES
 static struct shf	*shl_j;
-#endif /* JOBS */
-
-#ifdef NEED_PGRP_SYNC
-/* On some systems, the kernel doesn't count zombie processes when checking
- * if a process group is valid, which can cause problems in creating the
- * pipeline "cmd1 | cmd2": if cmd1 can die (and go into the zombie state)
- * before cmd2 is started, the kernel doesn't allow the setpgid() for cmd2
- * to succeed.  Solution is to create a pipe between the parent and the first
- * process; the first process doesn't do anything until the pipe is closed
- * and the parent doesn't close the pipe until all the processes are started.
- */
-static int		j_sync_pipe[2];
-static int		j_sync_open;
-#endif /* NEED_PGRP_SYNC */
-
-#ifdef TTY_PGRP
-static int		ttypgrp_ok;	/* set if can use tty pgrps */
+static bool		ttypgrp_ok;	/* set if can use tty pgrps */
 static pid_t		restore_ttypgrp = -1;
-static pid_t		our_pgrp;
 static int const	tt_sigs[] = { SIGTSTP, SIGTTIN, SIGTTOU };
-#endif /* TTY_PGRP */
+#endif
 
 static void		j_set_async(Job *);
 static void		j_startjob(Job *);
 static int		j_waitj(Job *, int, const char *);
-static RETSIGTYPE	j_sigchld(int);
+static void		j_sigchld(int);
 static void		j_print(Job *, int, struct shf *);
 static Job		*j_lookup(const char *, int *);
 static Job		*new_job(void);
@@ -181,20 +164,28 @@ static int		kill_job(Job *, int);
 
 /* initialize job control */
 void
-j_init(int UNUSED(mflagset))
+j_init(void)
 {
 	child_max = sysconf(_SC_CHILD_MAX);
+	if (child_max == -1)
+		child_max = 25;
+	Flag(FMONITOR) = 0;
 
-	sigemptyset(&sm_default);
+#ifdef JOBS_SIGS
+	(void)sigemptyset(&sm_default);
 	sigprocmask(SIG_SETMASK, &sm_default, NULL);
 
-	sigemptyset(&sm_sigchld);
-	sigaddset(&sm_sigchld, SIGCHLD);
+	(void)sigemptyset(&sm_sigchld);
+	(void)sigaddset(&sm_sigchld, SIGCHLD);
 
 	setsig(&sigtraps[SIGCHLD], j_sigchld,
-	    SS_RESTORE_ORIG|SS_FORCE|SS_SHTRAP);
+		SS_RESTORE_ORIG|SS_FORCE|SS_SHTRAP);
+#else /* JOB_SIGS */
+	/* Make sure SIGCHLD isn't ignored - can do odd things under SYSV */
+	setsig(&sigtraps[SIGCHLD], SIG_DFL, SS_RESTORE_ORIG|SS_FORCE);
+#endif /* JOB_SIGS */
 
-#ifdef JOBS
+#ifdef SILLY_FEATURES
 	if (!mflagset && Flag(FTALKING))
 		Flag(FMONITOR) = 1;
 
@@ -203,7 +194,6 @@ j_init(int UNUSED(mflagset))
 	 */
 	shl_j = shf_fdopen(2, SHF_WR, NULL);
 
-# ifdef TTY_PGRP
 	if (Flag(FMONITOR) || Flag(FTALKING)) {
 		int i;
 
@@ -217,13 +207,12 @@ j_init(int UNUSED(mflagset))
 			    SS_RESTORE_IGN|SS_FORCE);
 		}
 	}
-# endif /* TTY_PGRP */
 
 	/* j_change() calls tty_init() */
 	if (Flag(FMONITOR))
 		j_change();
 	else
-#endif /* JOBS */
+#endif
 	  if (Flag(FTALKING))
 		tty_init(TRUE);
 }
@@ -247,22 +236,21 @@ j_exit(void)
 				kill_job(j, SIGHUP);
 			else
 				killpg(j->pgrp, SIGHUP);
-#ifdef JOBS
+#ifdef SILLY_FEATURES
 			if (j->state == PSTOPPED) {
 				if (j->pgrp == 0)
 					kill_job(j, SIGCONT);
 				else
 					killpg(j->pgrp, SIGCONT);
 			}
-#endif /* JOBS */
+#endif
 		}
 	}
 	if (killed)
 		sleep(1);
 	j_notify();
 
-#ifdef JOBS
-# ifdef TTY_PGRP
+#ifdef SILLY_FEATURES
 	if (kshpid == procpid && restore_ttypgrp >= 0) {
 		/* Need to restore the tty pgrp to what it was when the
 		 * shell started up, so that the process that started us
@@ -274,15 +262,14 @@ j_exit(void)
 		tcsetpgrp(tty_fd, restore_ttypgrp);
 		setpgid(0, restore_ttypgrp);
 	}
-# endif /* TTY_PGRP */
 	if (Flag(FMONITOR)) {
 		Flag(FMONITOR) = 0;
 		j_change();
 	}
-#endif /* JOBS */
+#endif
 }
 
-#ifdef JOBS
+#ifdef SILLY_FEATURES
 /* turn job control on or off according to Flag(FMONITOR) */
 void
 j_change(void)
@@ -381,15 +368,14 @@ j_change(void)
 
 /* execute tree in child subprocess */
 int
-exchild(t, flags, close_fd)
-	struct op	*t;
-	int		flags;
-	int		close_fd;	/* used if XPCLOSE or XCCLOSE */
+exchild(struct op *t, int flags, volatile int *xerrok, int close_fd)
 {
 	static Proc	*last_proc;	/* for pipelines */
 
 	int		i;
+#ifdef JOB_SIGS
 	sigset_t	omask;
+#endif /* JOB_SIGS */
 	Proc		*p;
 	Job		*j;
 	int		rv = 0;
@@ -400,10 +386,12 @@ exchild(t, flags, close_fd)
 		/* Clear XFORK|XPCLOSE|XCCLOSE|XCOPROC|XPIPEO|XPIPEI|XXCOM|XBGND
 		 * (also done in another execute() below)
 		 */
-		return execute(t, flags & (XEXEC | XERROK));
+		return (execute(t, flags & (XEXEC | XERROK), xerrok));
 
 	/* no SIGCHLDs while messing with job and process lists */
+#ifdef JOB_SIGS
 	sigprocmask(SIG_BLOCK, &sm_sigchld, &omask);
+#endif
 
 	p = new_proc();
 	p->next = NULL;
@@ -418,20 +406,10 @@ exchild(t, flags, close_fd)
 			    "exchild: XPIPEI and no last_job - pid %d",
 			    (int)procpid);
 		j = last_job;
-		last_proc->next = p;
+		if (last_proc)
+			last_proc->next = p;
 		last_proc = p;
 	} else {
-#ifdef NEED_PGRP_SYNC
-		if (j_sync_open) {	/* should never happen */
-			j_sync_open = 0;
-			closepipe(j_sync_pipe);
-		}
-		/* don't do the sync pipe business if there is no pipeline */
-		if (flags & XPIPEO) {
-			openpipe(j_sync_pipe);
-			j_sync_open = 1;
-		}
-#endif /* NEED_PGRP_SYNC */
 		j = new_job(); /* fills in j->job */
 		/* we don't consider XXCOMs foreground since they don't get
 		 * tty process group and we don't save or restore tty modes.
@@ -445,7 +423,7 @@ exchild(t, flags, close_fd)
 		j->ppid = procpid;
 		j->age = ++njobs;
 		j->proc_list = p;
-#ifdef KSH
+#ifdef SILLY_FEATURES
 		j->coproc_id = 0;
 #endif /* KSH */
 		last_job = j;
@@ -466,13 +444,9 @@ exchild(t, flags, close_fd)
 	if (i < 0) {
 		kill_job(j, SIGKILL);
 		remove_job(j, "fork failed");
-#ifdef NEED_PGRP_SYNC
-		if (j_sync_open) {
-			closepipe(j_sync_pipe);
-			j_sync_open = 0;
-		}
-#endif /* NEED_PGRP_SYNC */
+#ifdef JOB_SIGS
 		sigprocmask(SIG_SETMASK, &omask, NULL);
+#endif
 		errorf("cannot fork - try again");
 	}
 	ischild = i == 0;
@@ -481,33 +455,10 @@ exchild(t, flags, close_fd)
 	else
 		p->pid = i;
 
-#ifdef JOBS
+#ifdef SILLY_FEATURES
 	/* job control set up */
 	if (Flag(FMONITOR) && !(flags&XXCOM)) {
 		int	dotty = 0;
-# ifdef NEED_PGRP_SYNC
-		int	first_child_sync = 0;
-# endif /* NEED_PGRP_SYNC */
-
-# ifdef NEED_PGRP_SYNC
-		if (j_sync_open) {
-			/*
-			 * The Parent closes 0, keeps 1 open 'til the whole
-			 * pipeline is started.  The First child closes 1,
-			 * keeps 0 open (reads from it).  The remaining
-			 * children just have to close 1 (parent has already
-			 * closeed 0).
-			 */
-			if (j->pgrp == 0) { /* First process */
-				close(j_sync_pipe[ischild]);
-				j_sync_pipe[ischild] = -1;
-				first_child_sync = ischild;
-			} else if (ischild) {
-				j_sync_open = 0;
-				closepipe(j_sync_pipe);
-			}
-		}
-# endif /* NEED_PGRP_SYNC */
 		if (j->pgrp == 0) {	/* First process */
 			j->pgrp = p->pid;
 			dotty = 1;
@@ -517,41 +468,26 @@ exchild(t, flags, close_fd)
 		 * condition
 		 */
 		setpgid(p->pid, j->pgrp);
-# ifdef TTY_PGRP
-		/* YYY: should this be
-		   if (ttypgrp_ok && ischild && !(flags&XBGND))
-			tcsetpgrp(tty_fd, j->pgrp);
-		   instead? (see also YYY below)
-		 */
 		if (ttypgrp_ok && dotty && !(flags & XBGND))
 			tcsetpgrp(tty_fd, j->pgrp);
-# endif /* TTY_PGRP */
-# ifdef NEED_PGRP_SYNC
-		if (first_child_sync) {
-			char c;
-			while (read(j_sync_pipe[0], &c, 1) == -1
-			       && errno == EINTR)
-				;
-			close(j_sync_pipe[0]);
-			j_sync_open = 0;
-		}
-# endif /* NEED_PGRP_SYNC */
 	}
-#endif /* JOBS */
+#endif
 
 	/* used to close pipe input fd */
-	if (close_fd >= 0 && (((flags & XPCLOSE) && !ischild)
-			      || ((flags & XCCLOSE) && ischild)))
+	if (close_fd >= 0 && (((flags & XPCLOSE) && !ischild) ||
+	    ((flags & XCCLOSE) && ischild)))
 		close(close_fd);
 	if (ischild) {		/* child */
-#ifdef KSH
+#ifdef SILLY_FEATURES
 		/* Do this before restoring signal */
 		if (flags & XCOPROC)
 			coproc_cleanup(FALSE);
 #endif /* KSH */
+#ifdef JOB_SIGS
 		sigprocmask(SIG_SETMASK, &omask, (sigset_t *) 0);
+#endif /* JOB_SIGS */
 		cleanup_parents_env();
-#ifdef TTY_PGRP
+#ifdef SILLY_FEATURES
 		/* If FMONITOR or FTALKING is set, these signals are ignored,
 		 * if neither FMONITOR nor FTALKING are set, the signals have
 		 * their inherited values.
@@ -575,14 +511,14 @@ exchild(t, flags, close_fd)
 		}
 		remove_job(j, "child");	/* in case of $(jobs) command */
 		nzombie = 0;
-#ifdef JOBS
+#ifdef SILLY_FEATURES
 		ttypgrp_ok = 0;
 		Flag(FMONITOR) = 0;
-#endif /* JOBS */
+#endif
 		Flag(FTALKING) = 0;
 		tty_close();
 		cleartraps();
-		execute(t, (flags & XERROK) | XEXEC); /* no return */
+		execute(t, (flags & XERROK) | XEXEC, NULL); /* no return */
 		internal_errorf(0, "exchild: execute() returned");
 		unwind(LLEAVE);
 		/* NOTREACHED */
@@ -592,14 +528,8 @@ exchild(t, flags, close_fd)
 	/* Ensure next child gets a (slightly) different $RANDOM sequence */
 	change_random();
 	if (!(flags & XPIPEO)) {	/* last process in a job */
-#ifdef TTY_PGRP
-		/* YYY: Is this needed? (see also YYY above)
-		   if (Flag(FMONITOR) && !(flags&(XXCOM|XBGND)))
-			tcsetpgrp(tty_fd, j->pgrp);
-		*/
-#endif /* TTY_PGRP */
 		j_startjob(j);
-#ifdef KSH
+#ifdef SILLY_FEATURES
 		if (flags & XCOPROC) {
 			j->coproc_id = coproc.id;
 			coproc.njobs++; /* n jobs using co-process output */
@@ -620,36 +550,44 @@ exchild(t, flags, close_fd)
 			rv = j_waitj(j, JW_NONE, "jw:last proc");
 	}
 
-	sigprocmask(SIG_SETMASK, &omask, (sigset_t *) 0);
+#ifdef JOB_SIGS
+	sigprocmask(SIG_SETMASK, &omask, NULL);
+#endif /* JOB_SIGS */
 
-	return rv;
+	return (rv);
 }
 
 /* start the last job: only used for $(command) jobs */
 void
 startlast(void)
 {
+#ifdef JOB_SIGS
 	sigset_t omask;
 
 	sigprocmask(SIG_BLOCK, &sm_sigchld, &omask);
+#endif /* JOB_SIGS */
 
 	if (last_job) { /* no need to report error - waitlast() will do it */
 		/* ensure it isn't removed by check_job() */
 		last_job->flags |= JF_WAITING;
 		j_startjob(last_job);
 	}
-	sigprocmask(SIG_SETMASK, &omask, (sigset_t *) 0);
+#ifdef JOB_SIGS
+	sigprocmask(SIG_SETMASK, &omask, NULL);
+#endif
 }
 
 /* wait for last job: only used for $(command) jobs */
 int
-waitlast()
+waitlast(void)
 {
 	int	rv;
 	Job	*j;
+#ifdef JOB_SIGS
 	sigset_t omask;
 
 	sigprocmask(SIG_BLOCK, &sm_sigchld, &omask);
+#endif /* JOB_SIGS */
 
 	j = last_job;
 	if (!j || !(j->flags & JF_STARTED)) {
@@ -657,15 +595,19 @@ waitlast()
 			warningf(TRUE, "waitlast: no last job");
 		else
 			internal_errorf(0, "waitlast: not started");
-		sigprocmask(SIG_SETMASK, &omask, (sigset_t *) 0);
-		return 125; /* not so arbitrary, non-zero value */
+#ifdef JOB_SIGS
+		sigprocmask(SIG_SETMASK, &omask, NULL);
+#endif /* JOB_SIGS */
+		return (125); /* not so arbitrary, non-zero value */
 	}
 
 	rv = j_waitj(j, JW_NONE, "jw:waitlast");
 
-	sigprocmask(SIG_SETMASK, &omask, (sigset_t *) 0);
+#ifdef JOB_SIGS
+	sigprocmask(SIG_SETMASK, &omask, NULL);
+#endif
 
-	return rv;
+	return (rv);
 }
 
 /* wait for child, interruptable. */
@@ -676,13 +618,15 @@ waitfor(const char *cp, int *sigp)
 	Job	*j;
 	int	ecode;
 	int	flags = JW_INTERRUPT|JW_ASYNCNOTIFY;
+#ifdef JOB_SIGS
 	sigset_t omask;
 
 	sigprocmask(SIG_BLOCK, &sm_sigchld, &omask);
+#endif /* JOB_SIGS */
 
 	*sigp = 0;
 
-	if (cp == (char *) 0) {
+	if (cp == NULL) {
 		/* wait for an unspecified job - always returns 0, so
 		 * don't have to worry about exited/signaled jobs
 		 */
@@ -691,32 +635,40 @@ waitfor(const char *cp, int *sigp)
 			if (j->ppid == procpid && j->state == PRUNNING)
 				break;
 		if (!j) {
-			sigprocmask(SIG_SETMASK, &omask, (sigset_t *) 0);
-			return -1;
+#ifdef JOB_SIGS
+			sigprocmask(SIG_SETMASK, &omask, NULL);
+#endif /* JOB_SIGS */
+			return (-1);
 		}
 	} else if ((j = j_lookup(cp, &ecode))) {
 		/* don't report normal job completion */
 		flags &= ~JW_ASYNCNOTIFY;
 		if (j->ppid != procpid) {
-			sigprocmask(SIG_SETMASK, &omask, (sigset_t *) 0);
-			return -1;
+#ifdef JOB_SIGS
+			sigprocmask(SIG_SETMASK, &omask, NULL);
+#endif /* JOB_SIGS */
+			return (-1);
 		}
 	} else {
-		sigprocmask(SIG_SETMASK, &omask, (sigset_t *) 0);
+#ifdef JOB_SIGS
+		sigprocmask(SIG_SETMASK, &omask, NULL);
+#endif /* JOB_SIGS */
 		if (ecode != JL_NOSUCH)
 			bi_errorf("%s: %s", cp, lookup_msgs[ecode]);
-		return -1;
+		return (-1);
 	}
 
 	/* at&t ksh will wait for stopped jobs - we don't */
 	rv = j_waitj(j, flags, "jw:waitfor");
 
-	sigprocmask(SIG_SETMASK, &omask, (sigset_t *) 0);
+#ifdef JOB_SIGS
+	sigprocmask(SIG_SETMASK, &omask, NULL);
+#endif /* JOB_SIGS */
 
 	if (rv < 0) /* we were interrupted */
 		*sigp = 128 + -rv;
 
-	return rv;
+	return (rv);
 }
 
 /* kill (built-in) a job */
@@ -726,14 +678,18 @@ j_kill(const char *cp, int sig)
 	Job	*j;
 	int	rv = 0;
 	int	ecode;
+#ifdef JOB_SIGS
 	sigset_t omask;
 
 	sigprocmask(SIG_BLOCK, &sm_sigchld, &omask);
+#endif /* JOB_SIGS */
 
-	if ((j = j_lookup(cp, &ecode)) == (Job *) 0) {
-		sigprocmask(SIG_SETMASK, &omask, (sigset_t *) 0);
+	if ((j = j_lookup(cp, &ecode)) == NULL) {
+#ifdef JOB_SIGS
+		sigprocmask(SIG_SETMASK, &omask, NULL);
+#endif /* JOB_SIGS */
 		bi_errorf("%s: %s", cp, lookup_msgs[ecode]);
-		return 1;
+		return (1);
 	}
 
 	if (j->pgrp == 0) {	/* started when !Flag(FMONITOR) */
@@ -742,7 +698,7 @@ j_kill(const char *cp, int sig)
 			rv = 1;
 		}
 	} else {
-#ifdef JOBS
+#ifdef SILLY_FEATURE
 		if (j->state == PSTOPPED && (sig == SIGTERM || sig == SIGHUP))
 			(void) killpg(j->pgrp, SIGCONT);
 #endif /* JOBS */
@@ -752,12 +708,14 @@ j_kill(const char *cp, int sig)
 		}
 	}
 
-	sigprocmask(SIG_SETMASK, &omask, (sigset_t *) 0);
+#ifdef JOB_SIGS
+	sigprocmask(SIG_SETMASK, &omask, NULL);
+#endif /* JOB_SIGS */
 
-	return rv;
+	return (rv);
 }
 
-#ifdef JOBS
+#ifdef SILLY_FEATURE
 /* fg and bg built-ins: called only if Flag(FMONITOR) set */
 int
 j_resume(const char *cp, int bg)
@@ -771,23 +729,23 @@ j_resume(const char *cp, int bg)
 
 	sigprocmask(SIG_BLOCK, &sm_sigchld, &omask);
 
-	if ((j = j_lookup(cp, &ecode)) == (Job *) 0) {
-		sigprocmask(SIG_SETMASK, &omask, (sigset_t *) 0);
+	if ((j = j_lookup(cp, &ecode)) == NULL) {
+		sigprocmask(SIG_SETMASK, &omask, NULL);
 		bi_errorf("%s: %s", cp, lookup_msgs[ecode]);
-		return 1;
+		return (1);
 	}
 
 	if (j->pgrp == 0) {
-		sigprocmask(SIG_SETMASK, &omask, (sigset_t *) 0);
+		sigprocmask(SIG_SETMASK, &omask, NULL);
 		bi_errorf("job not job-controlled");
-		return 1;
+		return (1);
 	}
 
 	if (bg)
 		shprintf("[%d] ", j->job);
 
 	running = 0;
-	for (p = j->proc_list; p != (Proc *) 0; p = p->next) {
+	for (p = j->proc_list; p != NULL; p = p->next) {
 		if (p->state == PSTOPPED) {
 			p->state = PRUNNING;
 			WSTATUS(p->status) = 0;
@@ -804,29 +762,31 @@ j_resume(const char *cp, int bg)
 	if (bg)
 		j_set_async(j);
 	else {
-# ifdef TTY_PGRP
 		/* attach tty to job */
 		if (j->state == PRUNNING) {
-			if (ttypgrp_ok && (j->flags & JF_SAVEDTTY)) {
-				set_tty(tty_fd, &j->ttystate, TF_NONE);
-			}
+			if (ttypgrp_ok && (j->flags & JF_SAVEDTTY))
+				tcsetattr(tty_fd, TCSADRAIN, &j->ttystate);
 			/* See comment in j_waitj regarding saved_ttypgrp. */
-			if (ttypgrp_ok && tcsetpgrp(tty_fd, (j->flags & JF_SAVEDTTYPGRP) ? j->saved_ttypgrp : j->pgrp) < 0) {
-				if (j->flags & JF_SAVEDTTY) {
-					set_tty(tty_fd, &tty_state, TF_NONE);
-				}
+			if (ttypgrp_ok &&
+			    tcsetpgrp(tty_fd, (j->flags & JF_SAVEDTTYPGRP) ?
+			    j->saved_ttypgrp : j->pgrp) < 0) {
+				rv = errno;
+				if (j->flags & JF_SAVEDTTY)
+					tcsetattr(tty_fd, TCSADRAIN, &tty_state);
 				sigprocmask(SIG_SETMASK, &omask,
-					(sigset_t *) 0);
+				    NULL);
 				bi_errorf("1st tcsetpgrp(%d, %d) failed: %s",
-					tty_fd, (int) ((j->flags & JF_SAVEDTTYPGRP) ? j->saved_ttypgrp : j->pgrp), strerror(errno));
-				return 1;
+				    tty_fd,
+				    (int)((j->flags & JF_SAVEDTTYPGRP) ?
+				    j->saved_ttypgrp : j->pgrp),
+				    strerror(rv));
+				return (1);
 			}
 		}
-# endif /* TTY_PGRP */
 		j->flags |= JF_FG;
 		j->flags &= ~JF_KNOWN;
 		if (j == async_job)
-			async_job = (Job *) 0;
+			async_job = NULL;
 	}
 
 	if (j->state == PRUNNING && killpg(j->pgrp, SIGCONT) < 0) {
@@ -834,61 +794,54 @@ j_resume(const char *cp, int bg)
 
 		if (!bg) {
 			j->flags &= ~JF_FG;
-# ifdef TTY_PGRP
-			if (ttypgrp_ok && (j->flags & JF_SAVEDTTY)) {
-				set_tty(tty_fd, &tty_state, TF_NONE);
-			}
-			if (ttypgrp_ok && tcsetpgrp(tty_fd, our_pgrp) < 0) {
-				warningf(TRUE,
-				"fg: 2nd tcsetpgrp(%d, %d) failed: %s",
-					tty_fd, (int) our_pgrp,
-					strerror(errno));
-			}
-# endif /* TTY_PGRP */
+			if (ttypgrp_ok && (j->flags & JF_SAVEDTTY))
+				tcsetattr(tty_fd, TCSADRAIN, &tty_state);
+			if (ttypgrp_ok && tcsetpgrp(tty_fd, kshpgrp) < 0)
+				warningf(true,
+				    "fg: 2nd tcsetpgrp(%d, %ld) failed: %s",
+				    tty_fd, (long)kshpgrp, strerror(errno));
 		}
-		sigprocmask(SIG_SETMASK, &omask, (sigset_t *) 0);
+		sigprocmask(SIG_SETMASK, &omask, NULL);
 		bi_errorf("cannot continue job %s: %s",
-			cp, strerror(err));
-		return 1;
+		    cp, strerror(err));
+		return (1);
 	}
 	if (!bg) {
-# ifdef TTY_PGRP
 		if (ttypgrp_ok) {
 			j->flags &= ~(JF_SAVEDTTY | JF_SAVEDTTYPGRP);
 		}
-# endif /* TTY_PGRP */
 		rv = j_waitj(j, JW_NONE, "jw:resume");
 	}
-	sigprocmask(SIG_SETMASK, &omask, (sigset_t *) 0);
-	return rv;
+	sigprocmask(SIG_SETMASK, &omask, NULL);
+	return (rv);
 }
 #endif /* JOBS */
 
 /* are there any running or stopped jobs ? */
 int
-j_stopped_running()
+j_stopped_running(void)
 {
 	Job	*j;
 	int	which = 0;
 
-	for (j = job_list; j != (Job *) 0; j = j->next) {
-#ifdef JOBS
+	for (j = job_list; j != NULL; j = j->next) {
+#ifdef SILLY_FEATURES
 		if (j->ppid == procpid && j->state == PSTOPPED)
 			which |= 1;
-#endif /* JOBS */
-		if (Flag(FLOGIN) && !Flag(FNOHUP) && procpid == kshpid
-		    && j->ppid == procpid && j->state == PRUNNING)
+#endif
+		if (Flag(FLOGIN) && !Flag(FNOHUP) && procpid == kshpid &&
+		    j->ppid == procpid && j->state == PRUNNING)
 			which |= 2;
 	}
 	if (which) {
 		shellf("You have %s%s%s jobs\n",
-			which & 1 ? "stopped" : "",
-			which == 3 ? " and " : "",
-			which & 2 ? "running" : "");
-		return 1;
+		    which & 1 ? "stopped" : "",
+		    which == 3 ? " and " : "",
+		    which & 2 ? "running" : "");
+		return (1);
 	}
 
-	return 0;
+	return (0);
 }
 
 #ifdef SILLY_FEATURES
@@ -905,7 +858,7 @@ j_njobs(void)
 		nj++;
 
 	sigprocmask(SIG_SETMASK, &omask, NULL);
-	return nj;
+	return (nj);
 }
 
 
@@ -917,9 +870,11 @@ j_jobs(const char *cp, int slp,
 	Job	*j, *tmp;
 	int	how;
 	int	zflag = 0;
+#ifdef JOB_SIGS
 	sigset_t omask;
 
 	sigprocmask(SIG_BLOCK, &sm_sigchld, &omask);
+#endif /* JOB_SIGS */
 
 	if (nflag < 0) { /* kludge: print zombies */
 		nflag = 0;
@@ -928,10 +883,12 @@ j_jobs(const char *cp, int slp,
 	if (cp) {
 		int	ecode;
 
-		if ((j = j_lookup(cp, &ecode)) == (Job *) 0) {
-			sigprocmask(SIG_SETMASK, &omask, (sigset_t *) 0);
+		if ((j = j_lookup(cp, &ecode)) == NULL) {
+#ifdef JOB_SIGS
+			sigprocmask(SIG_SETMASK, &omask, NULL);
+#endif /* JOB_SIGS */
 			bi_errorf("%s: %s", cp, lookup_msgs[ecode]);
-			return 1;
+			return (1);
 		}
 	} else
 		j = job_list;
@@ -952,21 +909,25 @@ j_jobs(const char *cp, int slp,
 		if (j->flags & JF_REMOVE)
 			remove_job(j, "jobs");
 	}
-	sigprocmask(SIG_SETMASK, &omask, (sigset_t *) 0);
-	return 0;
+#ifdef JOB_SIGS
+	sigprocmask(SIG_SETMASK, &omask, NULL);
+#endif /* JOB_SIGS */
+	return (0);
 }
 #endif /* SILLY_FEATURES */
 
 /* list jobs for top-level notification */
 void
-j_notify()
+j_notify(void)
 {
 	Job	*j, *tmp;
+#ifdef JOB_SIGS
 	sigset_t omask;
 
 	sigprocmask(SIG_BLOCK, &sm_sigchld, &omask);
+#endif /* JOB_SIGS */
 	for (j = job_list; j; j = j->next) {
-#ifdef JOBS
+#ifdef SILLY_FEATURES
 		if (Flag(FMONITOR) && (j->flags & JF_CHANGED))
 			j_print(j, JP_MEDIUM, shl_out);
 #endif /* JOBS */
@@ -982,23 +943,29 @@ j_notify()
 			remove_job(j, "notify");
 	}
 	shf_flush(shl_out);
+#ifdef JOB_SIGS
 	sigprocmask(SIG_SETMASK, &omask, NULL);
+#endif /* JOB_SIGS */
 }
 
 /* Return pid of last process in last asynchronous job */
 pid_t
 j_async(void)
 {
+#ifdef JOB_SIGS
 	sigset_t omask;
 
 	sigprocmask(SIG_BLOCK, &sm_sigchld, &omask);
+#endif /* JOB_SIGS */
 
 	if (async_job)
 		async_job->flags |= JF_KNOWN;
 
+#ifdef JOB_SIGS
 	sigprocmask(SIG_SETMASK, &omask, NULL);
+#endif /* JOB_SIGS */
 
-	return async_pid;
+	return (async_pid);
 }
 
 /* Make j the last async process
@@ -1050,17 +1017,13 @@ j_startjob(Job *j)
 		;
 	j->last_proc = p;
 
-#ifdef NEED_PGRP_SYNC
-	if (j_sync_open) {
-		j_sync_open = 0;
-		closepipe(j_sync_pipe);
-	}
-#endif /* NEED_PGRP_SYNC */
+#ifdef JOB_SIGS
 	if (held_sigchld) {
 		held_sigchld = 0;
 		/* Don't call j_sigchld() as it may remove job... */
 		kill(procpid, SIGCHLD);
 	}
+#endif /* JOB_SIGS */
 }
 
 /*
@@ -1090,7 +1053,11 @@ j_waitj(j, flags, where)
 		|| ((flags & JW_STOPPEDWAIT)
 		    && (volatile int) j->state == PSTOPPED))
 	{
+#ifdef JOB_SIGS
 		sigsuspend(&sm_default);
+#else /* JOB_SIGS */
+		j_sigchld(SIGCHLD);
+#endif /* JOB_SIGS */
 		if (fatal_trap) {
 			int oldf = j->flags & (JF_WAITING|JF_W_ASYNCNOTIFY);
 			j->flags &= ~(JF_WAITING|JF_W_ASYNCNOTIFY);
@@ -1099,7 +1066,7 @@ j_waitj(j, flags, where)
 		}
 		if ((flags & JW_INTERRUPT) && (rv = trap_pending())) {
 			j->flags &= ~(JF_WAITING|JF_W_ASYNCNOTIFY);
-			return -rv;
+			return (-rv);
 		}
 	}
 	j->flags &= ~(JF_WAITING|JF_W_ASYNCNOTIFY);
@@ -1108,7 +1075,7 @@ j_waitj(j, flags, where)
 		int	status;
 
 		j->flags &= ~JF_FG;
-#ifdef TTY_PGRP
+#ifdef SILLY_FEATURES
 		if (Flag(FMONITOR) && ttypgrp_ok && j->pgrp) {
 			/*
 			 * Save the tty's current pgrp so it can be restored
@@ -1167,7 +1134,7 @@ j_waitj(j, flags, where)
 					j->flags &= ~JF_USETTYMODE;
 			}
 		}
-#ifdef JOBS
+#ifdef SILLY_FEATURES
 		/* If it looks like user hit ^C to kill a job, pretend we got
 		 * one too to break out of for loops, etc.  (at&t ksh does this
 		 * even when not monitoring, but this doesn't make sense since
@@ -1195,14 +1162,15 @@ j_waitj(j, flags, where)
 	    && (!Flag(FMONITOR) || !(flags & JW_ASYNCNOTIFY)))
 		remove_job(j, where);
 
-	return rv;
+	return (rv);
 }
 
 /* SIGCHLD handler to reap children and update job states
  *
  * If jobs are compiled in then this routine expects sigchld to be blocked.
  */
-static RETSIGTYPE
+/* ARGSUSED */
+static void
 j_sigchld(int UNUSED(sig))
 {
 	int		errno_ = errno;
@@ -1212,6 +1180,7 @@ j_sigchld(int UNUSED(sig))
 	int		status;
 	struct rusage	ru0, ru1;
 
+#ifdef JOB_SIGS
 	/* Don't wait for any processes if a job is partially started.
 	 * This is so we don't do away with the process group leader
 	 * before all the processes in a pipe line are started (so the
@@ -1222,10 +1191,15 @@ j_sigchld(int UNUSED(sig))
 			held_sigchld = 1;
 			return;
 		}
+#endif /* JOB_SIGS */
 
 	getrusage(RUSAGE_CHILDREN, &ru0);
 	do {
+#ifdef JOB_SIGS
 		pid = waitpid(-1, &status, (WNOHANG|WUNTRACED));
+#else /* JOB_SIGS */
+		pid = wait(&status);
+#endif /* JOB_SIGS */
 
 		if (pid <= 0)	/* return if would block (0) ... */
 			break;	/* ... or no children or interrupted (-1) */
@@ -1253,7 +1227,7 @@ j_sigchld(int UNUSED(sig))
 		timersub(&j->systime, &ru0.ru_stime, &j->systime);
 		ru0 = ru1;
 		p->status = status;
-#ifdef JOBS
+#ifdef SILLY_FEATURES
 		if (WIFSTOPPED(status))
 			p->state = PSTOPPED;
 		else
@@ -1265,7 +1239,11 @@ j_sigchld(int UNUSED(sig))
 
 		check_job(j);	/* check to see if entire job is done */
 	}
+#ifdef JOB_SIGS
 	while (1);
+#else /* JOB_SIGS */
+	while (0);
+#endif /* JOB_SIGS */
 
 	errno = errno_;
 }
@@ -1312,7 +1290,7 @@ check_job(Job *j)
 		break;
 	}
 
-#ifdef KSH
+#ifdef SILLY_FEATURES
 	/* Note when co-process dies: can't be done in j_wait() nor
 	 * remove_job() since neither may be called for non-interactive
 	 * shells.
@@ -1338,7 +1316,7 @@ check_job(Job *j)
 #endif /* KSH */
 
 	j->flags |= JF_CHANGED;
-#ifdef JOBS
+#ifdef SILLY_FEATURES
 	if (Flag(FMONITOR) && !(j->flags & JF_XXCOM)) {
 		/* Only put stopped jobs at the front to avoid confusing
 		 * the user (don't want finished jobs effecting %+ or %-)
@@ -1408,7 +1386,7 @@ j_print(Job *j, int how, struct shf *shf)
 		return;
 	}
 	j->flags &= ~JF_CHANGED;
-	filler = j->job > 10 ?  "\n       " : "\n      ";
+	filler = j->job > 10 ? "\n       " : "\n      ";
 	if (j == job_list)
 		jobchar = '+';
 	else if (j == job_list->next)
@@ -1418,7 +1396,7 @@ j_print(Job *j, int how, struct shf *shf)
 		coredumped = 0;
 		switch (p->state) {
 		case PRUNNING:
-			strcpy(buf, "Running");
+			memcpy(buf, "Running", 8);
 			break;
 		case PSTOPPED:
 			strcpy(buf, sigtraps[WSTOPSIG(p->status)].mess);
@@ -1427,7 +1405,7 @@ j_print(Job *j, int how, struct shf *shf)
 			if (how == JP_SHORT)
 				buf[0] = '\0';
 			else if (WEXITSTATUS(p->status) == 0)
-				strcpy(buf, "Done");
+				memcpy(buf, "Done", 5);
 			else
 				snprintf(buf, sizeof(buf), "Done (%d)",
 					WEXITSTATUS(p->status));
@@ -1455,7 +1433,7 @@ j_print(Job *j, int how, struct shf *shf)
 		}
 
 		if (how == JP_LONG)
-			shf_fprintf(shf, "%5d ", p->pid);
+			shf_fprintf(shf, "%5d ", (int)p->pid);
 
 		if (how == JP_SHORT) {
 			if (buf[0]) {
@@ -1477,11 +1455,12 @@ j_print(Job *j, int how, struct shf *shf)
 		       && WSTATUS(p->status) == WSTATUS(status))
 		{
 			if (how == JP_LONG)
-				shf_fprintf(shf, "%s%5d %-20s %s%s", filler, p->pid,
-					' ', p->command, p->next ? "|" : null);
+				shf_fprintf(shf, "%s%5d %-20s %s%s", filler,
+				    (int)p->pid, " ", p->command,
+				    p->next ? "|" : null);
 			else if (how == JP_MEDIUM)
 				shf_fprintf(shf, " %s%s", p->command,
-					p->next ? "|" : null);
+				    p->next ? "|" : null);
 			p = p->next;
 		}
 	}
@@ -1505,27 +1484,27 @@ j_lookup(const char *cp, int *ecodep)
 		/* Look for last_proc->pid (what $! returns) first... */
 		for (j = job_list; j != NULL; j = j->next)
 			if (j->last_proc && j->last_proc->pid == job)
-				return j;
+				return (j);
 		/* ...then look for process group (this is non-POSIX,
 		 * but should not break anything) */
 		for (j = job_list; j != NULL; j = j->next)
 			if (j->pgrp && j->pgrp == job)
-				return j;
+				return (j);
 		if (ecodep)
 			*ecodep = JL_NOSUCH;
-		return NULL;
+		return (NULL);
 	}
 	if (*cp != '%') {
 		if (ecodep)
 			*ecodep = JL_INVALID;
-		return NULL;
+		return (NULL);
 	}
 	switch (*++cp) {
 	case '\0': /* non-standard */
 	case '+':
 	case '%':
 		if (job_list != NULL)
-			return job_list;
+			return (job_list);
 		break;
 
 	case '-':
@@ -1549,12 +1528,12 @@ j_lookup(const char *cp, int *ecodep)
 					if (last_match) {
 						if (ecodep)
 							*ecodep = JL_AMBIG;
-						return NULL;
+						return (NULL);
 					}
 					last_match = j;
 				}
 		if (last_match)
-			return last_match;
+			return (last_match);
 		break;
 
 	default:		/* %string */
@@ -1565,17 +1544,17 @@ j_lookup(const char *cp, int *ecodep)
 				if (last_match) {
 					if (ecodep)
 						*ecodep = JL_AMBIG;
-					return NULL;
+					return (NULL);
 				}
 				last_match = j;
 			}
 		if (last_match)
-			return last_match;
+			return (last_match);
 		break;
 	}
 	if (ecodep)
 		*ecodep = JL_NOSUCH;
-	return NULL;
+	return (NULL);
 }
 
 static Job	*free_jobs;
@@ -1606,7 +1585,7 @@ new_job(void)
 	}
 	newj->job = i;
 
-	return newj;
+	return (newj);
 }
 
 /* Allocate new process struct
@@ -1624,7 +1603,7 @@ new_proc(void)
 	} else
 		p = (Proc *) alloc(sizeof(Proc), APERM);
 
-	return p;
+	return (p);
 }
 
 /* Take job out of job_list and put old structures into free list.
@@ -1717,5 +1696,5 @@ kill_job(Job *j, int sig)
 		if (p->pid != 0)
 			if (kill(p->pid, sig) < 0)
 				rval = -1;
-	return rval;
+	return (rval);
 }
