@@ -85,50 +85,16 @@ const struct posh_option options[] = {
 	 * entries MUST match the order of sh_flag F* enumerations in sh.h.
 	 */
 	{ "allexport",	'a',		OF_ANY },
-#ifdef BRACE_EXPAND
-	{ "braceexpand",  0,		OF_ANY }, /* non-standard */
-#endif
 	{ (char *) 0, 	'c',	    OF_CMDLINE },
-#ifdef EMACS
-	{ "emacs",	  0,		OF_ANY },
-#endif
+	{ "noclobber",	'C',		OF_ANY },
 	{ "errexit",	'e',		OF_ANY },
-#ifdef EMACS
-	{ "gmacs",	  0,		OF_ANY },
-#endif
+	{ "noglob",	'f',		OF_ANY },
 	{ "ignoreeof",	  0,		OF_ANY },
 	{ "interactive",'i',	    OF_CMDLINE },
 	{ "login",	'l',	    OF_CMDLINE },
-#ifdef JOBS
-	{ "monitor",	'm',		OF_ANY },
-#else /* JOBS */
-	{ (char *) 0,	'm',		     0 }, /* so FMONITOR not ifdef'd */
-#endif /* JOBS */
-	{ "noclobber",	'C',		OF_ANY },
 	{ "noexec",	'n',		OF_ANY },
-	{ "noglob",	'f',		OF_ANY },
-	{ "nohup",	  0,		OF_ANY },
-	{ "nolog",	  0,		OF_ANY }, /* no effect */
-#ifdef	JOBS
-	{ "notify",	'b',		OF_ANY },
-#endif	/* JOBS */
 	{ "nounset",	'u',		OF_ANY },
-#ifdef SILLY_FEATURES
-	{ "posix",	  0,		OF_ANY }, /* non-standard */
-#endif /* SILLY_FEATURES */
-	{ "privileged",	'p',		OF_ANY },
-	{ "stdin",	's',	    OF_CMDLINE }, /* pseudo non-standard */
-#ifdef SILLY_FEATURES
-	{ "trackall",	'h',		OF_ANY },
-#endif /* SILLY_FEATURES */
 	{ "verbose",	'v',		OF_ANY },
-#ifdef VI
-	{ "vi",		  0,		OF_ANY },
-	{ "viraw",	  0,		OF_ANY }, /* no effect */
-	{ "vi-show8",	  0,		OF_ANY }, /* non-standard */
-	{ "vi-tabcomplete",  0, 	OF_ANY }, /* non-standard */
-	{ "vi-esccomplete",  0, 	OF_ANY }, /* non-standard */
-#endif
 	{ "xtrace",	'x',		OF_ANY },
 	/* Anonymous flags: used internally by shell only
 	 * (not visible to user)
