@@ -769,6 +769,7 @@ shf_fprintf(struct shf *shf, const char *fmt, ...)
 	return n;
 }
 
+#ifdef SILLY_FEATURES
 char *
 shf_smprintf(const char *fmt, ...)
 {
@@ -781,6 +782,7 @@ shf_smprintf(const char *fmt, ...)
 	va_end(args);
 	return shf_sclose(&shf); /* null terminates */
 }
+#endif /* SILLY_FEATURES */
 
 #undef FP  			/* if you want floating point stuff */
 
