@@ -34,7 +34,7 @@ static struct tbl *arraysearch  ARGS((struct tbl *, int));
  * assume caller has allocated and set up e->loc
  */
 void
-newblock()
+newblock(void)
 {
 	struct block *l;
 	static char *const empty[] = {null};
@@ -81,7 +81,7 @@ void popblock_walk(const void *nodep, const VISIT which, const int UNUSED(depth)
  * pop a block handling special variables
  */
 void
-popblock()
+popblock(void)
 {
 	struct block *l = e->loc;
 
@@ -95,7 +95,7 @@ popblock()
 
 /* called by main() to initialize variable data structures */
 void
-initvar()
+initvar(void)
 {
 	static const struct {
 		const char *name;
@@ -167,7 +167,7 @@ array_index_calc(n, arrayp, valp)
 		*valp = rval;
 		afree(sub, ATEMP);
 	}
-	return n;
+	return (n);
 }
 
 /*
@@ -228,15 +228,15 @@ global(const char *n)
 		  default:
 			vp->flag &= ~(ISSET|INTEGER);
 		}
-		return vp;
+		return (vp);
 	}
 	for (l = e->loc; ; l = l->next) {
 		vp = transitional_tsearch(&l->vars.root, n);
 		if (vp != NULL) {
 			if (array)
-				return arraysearch(vp, val);
+				return (arraysearch(vp, val));
 			else
-				return vp;
+				return (vp);
 		}
 		if (l->next == NULL)
 			break;
@@ -247,7 +247,7 @@ global(const char *n)
 	vp->flag |= DEFINED;
 	if (special(n))
 		vp->flag |= SPECIAL;
-	return vp;
+	return (vp);
 }
 
 /*
@@ -256,8 +256,8 @@ global(const char *n)
 struct tbl *
 local(const char *n, bool_t copy)
 {
-	register struct block *l = e->loc;
-	register struct tbl *vp;
+	struct block *l = e->loc;
+	struct tbl *vp;
 	bool_t	 array;
 	int	 val;
 
@@ -268,7 +268,7 @@ local(const char *n, bool_t copy)
 		vp->flag = DEFINED|RDONLY;
 		vp->type = 0;
 		vp->areap = ATEMP;
-		return vp;
+		return (vp);
 	}
 	vp = transitional_tenter(&l->vars.root, n, &l->area);
 	if (copy && !(vp->flag & DEFINED)) {
@@ -289,13 +289,12 @@ local(const char *n, bool_t copy)
 	vp->flag |= DEFINED;
 	if (special(n))
 		vp->flag |= SPECIAL;
-	return vp;
+	return (vp);
 }
 
 /* get variable string value */
 char *
-str_val(vp)
-	register struct tbl *vp;
+str_val(struct tbl *vp)
 {
 	char *s;
 
@@ -346,15 +345,12 @@ intval(vp)
 	if (base == -1)
 		/* XXX check calls - is error here ok by POSIX? */
 		errorf("%s: bad number", str_val(vp));
-	return num;
+	return (num);
 }
 
 /* set variable to string value */
 int
-setstr(vq, s, error_ok)
-	register struct tbl *vq;
-	const char *s;
-	int error_ok;
+setstr(struct tbl *vq, const char *s, int error_ok)
 {
 	int no_ro_check = error_ok & 0x4;
 	error_ok &= ~0x4;
@@ -362,7 +358,7 @@ setstr(vq, s, error_ok)
 		warningf(TRUE, "%s: is read only", vq->name);
 		if (!error_ok)
 			errorf(null);
-		return 0;
+		return (0);
 	}
 	if (!(vq->flag&INTEGER)) { /* string dest */
 		if ((vq->flag&ALLOC)) {
@@ -389,17 +385,15 @@ setstr(vq, s, error_ok)
 	vq->flag |= ISSET;
 	if ((vq->flag&SPECIAL))
 		setspec(vq);
-	return 1;
+	return (1);
 }
 
 /* set variable to integer */
 void
-setint(vq, n)
-	register struct tbl *vq;
-	long n;
+setint(struct tbl *vq, long n)
 {
 	if (!(vq->flag&INTEGER)) {
-		register struct tbl *vp = &vtemp;
+		struct tbl *vp = &vtemp;
 		vp->flag = (ISSET|INTEGER);
 		vp->type = 0;
 		vp->areap = ATEMP;
@@ -414,24 +408,21 @@ setint(vq, n)
 }
 
 int
-getint(vp, nump)
-	struct tbl *vp;
-	long *nump;
+getint(struct tbl *vp, long *nump)
 {
-	register char *s;
-	register int c;
-	int base, neg;
+	char *s;
+	int c, base, neg;
 	int have_base = 0;
 	long num;
-	
+
 	if (vp->flag&SPECIAL)
 		getspec(vp);
 	/* XXX is it possible for ISSET to be set and val.s to be 0? */
 	if (!(vp->flag&ISSET) || (!(vp->flag&INTEGER) && vp->val.s == NULL))
-		return -1;
+		return (-1);
 	if (vp->flag&INTEGER) {
 		*nump = vp->val.i;
-		return vp->type;
+		return (vp->type);
 	}
 	s = vp->val.s + vp->type;
 	if (s == NULL)	/* redundent given initial test */
@@ -529,11 +520,9 @@ formatstr(vp, s)
  * make vp->val.s be "name=value" for quick exporting.
  */
 static void
-export(vp, val)
-	register struct tbl *vp;
-	const char *val;
+export(struct tbl *vp, const char *val)
 {
-	register char *xp;
+	char *xp;
 	char *op = (vp->flag&ALLOC) ? vp->val.s : NULL;
 	int namelen = strlen(vp->name);
 	int vallen = strlen(val) + 1;
@@ -565,13 +554,13 @@ typeset(const char *var, Tflag set, Tflag clr, int UNUSED(field), int base)
 	/* check for valid variable name, search for value */
 	val = skip_varname(var, FALSE);
 	if (val == var)
-		return NULL;
+		return (NULL);
 	if (*val == '[') {
 		int len;
 		
 		len = array_ref_len(val);
 		if (len == 0)
-			return NULL;
+			return (NULL);
 		/* IMPORT is only used when the shell starts up and is
 		 * setting up its environment.  Allow only simple array
 		 * references at this time since parameter/command substitution
@@ -691,16 +680,14 @@ typeset(const char *var, Tflag set, Tflag clr, int UNUSED(field), int base)
 	    && vpbase->type == 0)
 		export(vpbase, (vpbase->flag&ISSET) ? vpbase->val.s : null);
 
-	return vp;
+	return (vp);
 }
 
 /* Unset a variable.  array_ref is set if there was an array reference in
  * the name lookup (eg, x[2]).
  */
 void
-unset(vp, array_ref)
-	register struct tbl *vp;
-	int array_ref;
+unset(struct tbl *vp, int array_ref)
 {
 	if (vp->flag & ALLOC)
 		afree((void*)vp->val.s, vp->areap);
@@ -728,9 +715,7 @@ unset(vp, array_ref)
  * null if whole string is legal).
  */
 char *
-skip_varname(s, aok)
-	const char *s;
-	int aok;
+skip_varname(const char *s, int aok)
 {
 	int alen;
 
@@ -745,9 +730,7 @@ skip_varname(s, aok)
 
 /* Return a pointer to the first character past any legal variable name.  */
 char *
-skip_wdvarname(s, aok)
-	const char *s;
-	int aok;	/* skip array de-reference? */
+skip_wdvarname(const char *s, int aok)
 {
 	if (s[0] == CHAR && (isalpha(s[1]) || s[1]=='_')) {
 		do
@@ -778,9 +761,7 @@ skip_wdvarname(s, aok)
 
 /* Check if coded string s is a variable name */
 int
-is_wdvarname(s, aok)
-	const char *s;
-	int aok;
+is_wdvarname(const char *s, int aok)
 {
 	char *p = skip_wdvarname(s, aok);
 
@@ -789,8 +770,7 @@ is_wdvarname(s, aok)
 
 /* Check if coded string s is a variable assignment */
 int
-is_wdvarassign(s)
-	const char *s;
+is_wdvarassign(const char *s)
 {
 	char *p = skip_wdvarname(s, TRUE);
 
@@ -932,8 +912,7 @@ getspec(vp)
 }
 
 static void
-setspec(vp)
-	register struct tbl *vp;
+setspec(struct tbl *vp)
 {
 	char *s;
 
@@ -1029,8 +1008,7 @@ setspec(vp)
 }
 
 static void
-unsetspec(vp)
-	register struct tbl *vp;
+unsetspec(struct tbl *vp)
 {
 	switch (special(vp->name)) {
 	  case V_PATH:
@@ -1085,9 +1063,7 @@ unsetspec(vp)
  * vp, indexed by val.
  */
 static struct tbl *
-arraysearch(vp, val)
-	struct tbl *vp;
-	int val;
+arraysearch(struct tbl *vp, int val)
 {
 	struct tbl *prev, *curr, *new;
 
@@ -1121,7 +1097,7 @@ arraysearch(vp, val)
 		prev->u.array = new;
 		new->u.array = curr;
 	}
-	return new;
+	return (new);
 }
 
 /* Return the length of an array reference (eg, [1+2]) - cp is assumed
@@ -1129,8 +1105,7 @@ arraysearch(vp, val)
  * bracket.
  */
 int
-array_ref_len(cp)
-	const char *cp;
+array_ref_len(const char *cp)
 {
 	const char *s = cp;
 	int c;
@@ -1140,16 +1115,15 @@ array_ref_len(cp)
 		if (c == '[')
 			depth++;
 	if (!c)
-		return 0;
-	return s - cp;
+		return (0);
+	return (s - cp);
 }
 
 /*
  * Make a copy of the base of an array name
  */
 char *
-arrayname(str)
-	const char *str;
+arrayname(const char *str)
 {
 	const char *p;
 
